@@ -1,3 +1,8 @@
+
+from generallibrary import deco_cache
+from generalfile.errors import CaseSensitivityError
+from generalfile.decorators import deco_require_state, deco_preserve_working_dir, deco_return_if_removed
+
 import pathlib
 import appdirs
 import os
@@ -7,81 +12,7 @@ import json
 from distutils.dir_util import copy_tree
 import time
 
-from generallibrary import deco_cache
-
-from generalfile.errors import CaseSensitivityError
-from generalfile.decorators import deco_require_state, deco_preserve_working_dir, deco_return_if_removed
-
-
-
-class _Context:
-    def __init__(self, path):
-        self.path = path
-        self.temp_path = self.path.with_suffix(".tmp")
-        self.lock = self.path.lock()
-
-    def __enter__(self):
-        self.lock.__enter__()
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        self.lock.__exit__(exc_type, exc_val, exc_tb)
-
-
-class WriteContext(_Context):
-    """ A context manager used for every write method. """
-    def __init__(self, path, overwrite=False):
-        super().__init__(path)
-        self.overwrite = overwrite
-
-    def __enter__(self):
-        """ :rtype: generalfile.Path """
-        if not self.overwrite and self.path.exists():
-            raise FileExistsError(f"Path '{self.path}' already exists and overwrite is 'False'.")
-        super().__enter__()
-        self.path.get_parent().create_folder()
-        return self.temp_path
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        self.temp_path.rename(self.path.name(), overwrite=True)
-        super().__exit__(exc_type, exc_val, exc_tb)
-
-
-class ReadContext(_Context):
-    """ A context manager used for every read method. """
-    def __init__(self, path):
-        super().__init__(path)
-
-    def __enter__(self):
-        super().__enter__()
-        return self.path
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        super().__exit__(exc_type, exc_val, exc_tb)
-
-
-class AppendContext(_Context):
-    """ A context manager used for every append method. """
-    def __init__(self, path):
-        super().__init__(path)
-
-    def __enter__(self):
-        super().__enter__()
-        if self.path.exists():
-            self.path.copy(self.temp_path)
-        return self.temp_path
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        self.temp_path.rename(self.path.name(), overwrite=True)
-        super().__exit__(exc_type, exc_val, exc_tb)
-
-
-class _Extension:
-    WriteContext = WriteContext
-    ReadContext = ReadContext
-    AppendContext = AppendContext
-
-    def __init__(self, path):
-        self.path = path
+from generalfile.optional_dependencies._extension import WriteContext, ReadContext
 
 
 class Path_Operations:
@@ -91,19 +22,29 @@ class Path_Operations:
     dead_lock_seconds = 3
     _working_dir = None
 
-    def open_operation(self, mode, func):
-        """ Handles all open() calls. """
-        with open(str(self), mode, encoding="utf-8") as stream:
-            return func(stream)
+    def _removed_path(self):
+        """ :param generalfile.Path self: """
+        self.set_parent(None)
 
-        # Couldn't do this as it only fails when writing, not reading
-        # try:
-        #     with open(str(self), mode) as stream:
-        #         return func(stream)
-        # except UnicodeEncodeError:
-        #     with open(str(self), mode, encoding="utf-8") as stream:
-        #         return func(stream)
+    def open_operation(self, mode, func, encoding=..., no_file_default=...):
+        """ Handles all open() calls.
 
+            :param generalfile.Path self:
+            :param mode:
+            :param func:
+            :param encoding:
+            :param no_file_default: """
+        if encoding is Ellipsis:
+            encoding = "utf-8"
+
+        try:
+            with open(self, mode, encoding=encoding) as stream:
+                return func(stream)
+        except FileNotFoundError as e:
+            if no_file_default is ...:
+                raise e
+            else:
+                return no_file_default
 
     def write(self, content=None, overwrite=False, indent=None):
         """ Write to this Path with JSON.
@@ -112,7 +53,7 @@ class Path_Operations:
             :param any content: Serializable by JSON
             :param overwrite: Whether to allow overwriting or not.
             :param indent: """
-        content_json = json.dumps(content, indent=indent)
+        content_json = "" if content is None else json.dumps(content, indent=indent)
         with WriteContext(self, overwrite=overwrite) as write_path:
             write_path.open_operation("w", lambda stream: stream.write(content_json))
         return content_json
@@ -123,13 +64,7 @@ class Path_Operations:
             :param generalfile.Path self:
             :param default: Optionally return a default value. """
         with ReadContext(self) as read_path:
-            try:
-                return read_path.open_operation("r", lambda stream: json.loads(stream.read()))
-            except FileNotFoundError as e:
-                if default is ...:
-                    raise e
-                else:
-                    return default
+            return read_path.open_operation("r", lambda stream: json.loads(stream.read()), no_file_default=default)
 
     @deco_require_state(exists=True)
     def rename(self, name=None, stem=None, suffix=None, overwrite=False):
@@ -149,9 +84,11 @@ class Path_Operations:
 
         with self.lock(new_path):
             if overwrite:
-                self._path.replace(str(new_path))
+                self._path.replace(new_path)
             else:
-                self._path.rename(str(new_path))
+                self._path.rename(new_path)
+            self._removed_path()
+        return new_path
 
     @deco_require_state(exists=True)
     def copy(self, new_path, overwrite=False):
@@ -174,10 +111,11 @@ class Path_Operations:
             self._copy_file_or_folder(new_path=new_path)
 
     def _copy_file_or_folder(self, new_path):
+        """ :param generalfile.Path self: """
         if self.is_file():
-            shutil.copy(str(self), str(new_path), follow_symlinks=False)  # Can clobber
+            shutil.copy(self.path, new_path, follow_symlinks=False)  # Can clobber
         else:
-            copy_tree(str(self), str(new_path))
+            copy_tree(self.path, new_path.path)
 
     @deco_require_state(exists=True)
     def _copy_or_move(self, target_folder_path, overwrite, method):
@@ -191,12 +129,12 @@ class Path_Operations:
             return
 
         if self.is_file():
-            filepaths = (self,)
+            filepaths = [self]
         else:
-            filepaths = tuple(self.get_paths_in_folder())
+            filepaths = self.get_children()
 
         target_filepaths = [target_folder_path / path.absolute().relative(self_parent_path) for path in filepaths]
-        if not overwrite and any([target.exists(quick=True) for target in target_filepaths]):
+        if not overwrite and any([target.exists() for target in target_filepaths]):
             raise FileExistsError("Atleast one target filepath exists, cannot copy")
 
         with self.lock(target_folder_path):
@@ -207,10 +145,12 @@ class Path_Operations:
                     self.__class__._copy_file_or_folder(path, target)  # Same as path._copy_file_or_folder(target)
 
                 elif method == "move":
-                    shutil.move(str(path), str(target))  # Can clobber if full target path is specified like we do
+                    shutil.move(path, target)  # Can clobber if full target path is specified like we do
 
-            if method == "move" and self.is_folder():
-                self.delete()
+            if method == "move":
+                self._removed_path()
+                if self.is_folder():
+                    self.delete()
 
     def copy_to_folder(self, target_folder_path, overwrite=False):
         """ Copy file or files inside given folder to anything except it's own parent, use `copy` for that.
@@ -240,25 +180,38 @@ class Path_Operations:
             :param generalfile.Path self: """
         return self._path.is_dir()
 
-    def exists(self, quick=False):
+    def is_root(self):
+        """ Get whether this Path is a root.
+
+            :param generalfile.Path self: """
+        if self.verInfo.pathRootIsDelimiter:
+            return self.path == self.path_delimiter
+        else:
+            return len(self.path) == 3 and self.path[1] == ":" and self.path[2] == self.path_delimiter
+
+    def _case_sens_test(self, path):
+        """ :param generalfile.Path self: """
+        return self != path and self.path.lower() == str(path).lower()
+
+    def exists(self):
         """ Get whether this Path exists.
 
-            :param generalfile.Path self:
-            :param quick: Whether to do a quick (case insensitive on windows) check. """
-        if quick:
-            return self._path.exists()
-        else:
-            try:
-                path_list = self.get_paths_recursive(depth=0, include_self=True)
-            except AttributeError:
-                return False
-            exists = False
-            for foundPath in path_list:
-                if foundPath == self:
-                    exists = True
-                elif str(foundPath).lower() == str(self).lower():
-                    raise CaseSensitivityError(f"Same path with differing case not allowed: '{self}'")
-            return exists
+            :param generalfile.Path self: """
+        if self.get_sibling(spawn=False, filt=self._case_sens_test, traverse_excluded=True):
+            raise CaseSensitivityError(f"Same path with differing case not allowed: '{self}'")
+        return self._path.exists()
+
+    def empty(self):
+        """ Get whether path is an empty folder or not.
+
+            :param generalfile.Path self: """
+        if not self.exists():
+            return True
+        elif self.is_file():
+            return False
+        if self.get_child():
+            return False
+        return True
 
     def without_file(self):
         """ Get this path without it's name if it's a file, otherwise it returns itself.
@@ -269,58 +222,6 @@ class Path_Operations:
         else:
             return self
 
-    @deco_require_state(is_folder=True)
-    def get_paths_in_folder(self, relative=None):
-        """ Get a generator containing every child Path inside this folder, relative if possible.
-
-            :param generalfile.Path self:
-            :param relative: """
-        for child in self._path.iterdir():
-            if relative is None:
-                yield self.Path(child, parent=self)
-            else:
-                yield self.Path(child).relative(base=relative).set_parent(parent=self)
-
-    @deco_require_state(quick_exists=True)
-    def get_paths_recursive(self, depth=-1, include_self=False, include_files=True, include_folders=False, relative=None, filt=None):
-        """ Get all paths that are next to this file or inside this folder.
-
-            :param depth: Depth of -1 is limitless recursive searching. Depth of 1 searches only first level.
-            :param include_self:
-            :param include_files:
-            :param include_folders:
-            :param generalfile.Path self:
-            :param relative:
-            :param function filt: Optional filter with Path as arg. """
-        if self.is_file():
-            queued_folders = [self.get_parent()]
-        elif self.is_folder():
-            queued_folders = [self]
-        else:
-            raise AttributeError(f"Path {self} is neither file nor folder.")
-
-        self_parts_len = len(queued_folders[0].parts())
-
-        if include_self:
-            yield self if relative is None else self.relative(base=relative)
-
-        while queued_folders:
-            for path in queued_folders[0].get_paths_in_folder():
-                if filt and not filt(path):
-                    continue
-
-                if path.is_file():
-                    if include_files and path != self:
-                        yield path if relative is None else path.relative(base=relative)
-                elif path.is_folder():
-                    if include_folders:
-                        yield path if relative is None else path.relative(base=relative)
-
-                    current_depth = len(path.parts()) - self_parts_len + 1
-                    if depth == -1 or current_depth < depth:
-                        queued_folders.append(path)
-            del queued_folders[0]
-
     def create_folder(self):
         """ Create folder with this Path unless it exists. 
 
@@ -336,23 +237,7 @@ class Path_Operations:
 
             :param generalfile.Path self: """
         self.create_folder()
-        os.startfile(str(self.without_file()))
-
-    @classmethod
-    @deco_cache()
-    def get_cache_dir(cls):
-        """ Get cache folder.
-
-            :param generalfile.Path cls: """
-        return cls.Path(appdirs.user_cache_dir())
-
-    @classmethod
-    @deco_cache()
-    def get_lock_dir(cls):
-        """ Get lock folder inside cache folder.
-
-            :param generalfile.Path cls: """
-        return cls.Path.get_cache_dir() / "generalfile" / "locks"
+        os.startfile(self.without_file())
 
     @classmethod
     def get_working_dir(cls):
@@ -380,26 +265,56 @@ class Path_Operations:
             :param generalfile.Path self: """
         self.create_folder()
         self._working_dir = self.absolute()
-        os.chdir(str(self._working_dir))
+        os.chdir(self._working_dir)
+
+    @classmethod
+    @deco_cache()
+    def get_cache_dir(cls):
+        """ Get cache folder.
+
+            :param generalfile.Path or Any cls:
+            :rtype: generalfile.Path """
+        path = cls(appdirs.user_cache_dir())
+        return path
+
+    @classmethod
+    @deco_cache()
+    def get_lock_dir(cls):
+        """ Get lock folder inside cache folder.
+
+            :param generalfile.Path cls:
+            :rtype: generalfile.Path """
+        return cls.get_cache_dir() / "generalfile" / "locks"
+
+    @deco_cache()
+    def get_lock_path(self):
+        """ Get absolute lock path pointing to actual lock.
+
+            :param generalfile.Path self:
+            :rtype: generalfile.Path """
+        return self.get_lock_dir() / self.absolute().to_alternative()
 
     @deco_preserve_working_dir
     @deco_return_if_removed(content=False)
     def delete(self, error=True):
         """ Delete a file or folder.
-            Todo: Proper decorator to optionally suppress specified errors.
 
             :param error:
             :param generalfile.Path self: """
         with self.lock():
             try:
                 if self.is_file():
-                    os.remove(str(self))
+                    os.remove(self)
                 elif self.is_folder():
-                    shutil.rmtree(str(self), ignore_errors=True)
+                    shutil.rmtree(self)
+            except PermissionError:
+                self.trash()  # Sometimes failing to remove a git file on windows, send2trash still worked
             except Exception as e:
                 if error:
                     raise e
 
+            self._removed_path()
+
     @deco_preserve_working_dir
     @deco_return_if_removed(content=False)
     def trash(self):
@@ -407,15 +322,17 @@ class Path_Operations:
 
             :param generalfile.Path self: """
         with self.lock():
-            send2trash(str(self))
+            send2trash(self.path)
+
+            self._removed_path()
 
     @deco_preserve_working_dir
     @deco_return_if_removed(content=True)
     def delete_folder_content(self):
-        """ Delete a file or folder and then create an empty folder in it's place.
+        """ Delete every path in a folder.
 
             :param generalfile.Path self: """
-        for path in self.get_paths_in_folder():
+        for path in self.get_children(gen=True):
             path.delete()
 
     @deco_preserve_working_dir
@@ -424,7 +341,7 @@ class Path_Operations:
         """ Trash a file or folder and then create an empty folder in it's place.
 
             :param generalfile.Path self: """
-        for path in self.get_paths_in_folder():
+        for path in self.get_children(gen=True):
             path.trash()
 
     @deco_require_state(is_file=True)
@@ -433,18 +350,14 @@ class Path_Operations:
             NOTE: Doesn't seem to update very quickly for windows (7).
 
             :param generalfile.Path self: """
-        return time.time() - os.path.getctime(str(self))
+        return time.time() - os.path.getctime(self)
 
     @deco_require_state(is_file=True)
     def seconds_since_modified(self):
         """ Get time in seconds since file was modified.
 
             :param generalfile.Path self: """
-        return time.time() - os.path.getmtime(str(self))
-
-
-
-
+        return time.time() - os.path.getmtime(self)
 
     @deco_require_state(is_file=True)
     def size(self):
@@ -454,62 +367,101 @@ class Path_Operations:
         return self._path.stat().st_size
 
     def is_identical(self, path):
-        """ Get whether this file is identical to another.
+        """ Get whether this file's content is identical to another.
 
             :param generalfile.Path self:
             :param path: """
         path = self.Path(path)
-        self_exists = self.exists(quick=True)
-        path_exists = path.exists(quick=True)
+        self_exists = self.exists()
+        path_exists = path.exists()
 
         if not self_exists or not path_exists:
             return self_exists == path_exists
 
-        # if self.size() != path.size():
-        #     return False
-
         with self.lock(path):
-            with open(str(self), "r") as file1:
-                with open(str(path), "r") as file2:
-                    # print(file1.read())
-                    # print(file2.read())
+            with open(self, "r") as file1:
+                with open(path, "r") as file2:
                     return file1.read() == file2.read()
 
     @deco_require_state(is_folder=True)
     def get_differing_files(self, target, exist=True, content=True, filt=None):
-        """ Get list of changed files by comparing two folders.
+        """ Get a set of changed files by comparing two folders.
 
             :param generalfile.Path self:
             :param target:
-            :param exist:
-            :param content:
-            :param filt: """
+            :param exist: Whether to compare files' existence. Ignores content.
+            :param content: Whether to compare files' content if both files exist.
+            :param filt: Optional filter, takes one Path as arg. """
         target = self.Path(target)
         assert target.is_folder()
 
-        self_paths = set(self.get_paths_recursive(relative=self, filt=filt))
-        target_paths = set(target.get_paths_recursive(relative=target, filt=filt))
+        if filt is None:
+            new_filt = lambda path: path.is_file()
+        else:
+            new_filt = lambda path: path.is_file() and filt(path)
+
+        self_paths = {child.relative(self) for child in self.get_children(depth=-1, filt=new_filt, traverse_excluded=True)}
+        target_paths = {child.relative(target) for child in target.get_children(depth=-1, filt=new_filt, traverse_excluded=True)}
 
         diff = set()
         if exist:
-            diff.update(self_paths.difference(target_paths))
+            diff.update(self_paths.symmetric_difference(target_paths))
         if content:
-            diff.update({path for path in self_paths.intersection(target_paths) if not (self / path).is_identical(path=target / path)})
+            diff.update({path for path in self_paths.intersection(target_paths) if path not in diff and not (self / path).is_identical(path=target / path)})
         return diff
 
+    @deco_require_state(is_file=True)
     def contains(self, text):
-        """ Return whether text string exists in one of the files.
+        """ Return whether text string exists in one of the lines in this file.
 
-            :param paths tuple[generalfile.Path] self:
+            :param generalfile.Path self:
             :param text: """
         with self.lock():
-            with open(str(self), "r") as stream:
+            with open(self, "r") as stream:
                 for line in stream:
                     if text in line:
                         return True
         return False
 
+    def _pack_default_suffix(self):
+        """ :param generalfile.Path self: """
+        if not self.suffix():
+            return self.with_suffix(".zip")
+        else:
+            return self
 
+    @deco_require_state(is_folder=True)
+    def pack(self, target, overwrite=False):
+        """ Pack self which is folder to a new target archive.
+
+            :param generalfile.Path self: Base folder to be packed.
+            :param target: Full path of new archive. Optional suffix, defaults to zip if missing.'
+            :param overwrite: """
+        target = self.Path(target)._pack_default_suffix().absolute()
+        if not overwrite:
+            assert not target.exists()
+
+        root_dir = self.absolute()
+        target_stem = str(target.with_suffixes())
+        target_suffix = "".join(target.suffixes())[1:]
+        target_suffix = {"tar.gz": "gztar"}.get(target_suffix, target_suffix)
+
+        shutil.make_archive(root_dir=root_dir, base_name=target_stem, format=target_suffix)
+
+        return target
+
+    def unpack(self, base, overwrite=False):
+        """ Unpack self which is archive to target folder (Must be empty if overwrite is False).
+
+            :param generalfile.Path self:
+            :param base:
+            :param overwrite: """
+        base = self.Path(base)
+        if not overwrite:
+            assert not base.exists() or base.empty()
+
+        shutil.unpack_archive(filename=self._pack_default_suffix(), extract_dir=base)
+        return base
 
 
 
