@@ -2,8 +2,8 @@
 from generalfile import Path
 from generalfile.test.test_path import PathTest
 
-class FileTest(PathTest):
 
+class FileTest(PathTest):
     def test_cfg(self):
         dict_ = {'test': {'foo': 'bar', 'number': 2, 'hi': ['a', 'b', 3]}}
 
@@ -23,4 +23,7 @@ class FileTest(PathTest):
 
         self.assertEqual({'setup': {'foo': 'random'}, 'test': {'5': 'bar'}}, Path("foo").cfg.read())
 
-
+    def test_read_empty(self):
+        with self.assertRaises(FileNotFoundError):
+            Path("hey").cfg.read()
+        self.assertEqual(None, Path("hi").cfg.read(default=None))
