@@ -2,19 +2,22 @@
 import unittest
 import multiprocessing as mp
 
-from generalfile import Path
+from generalfile import *
 from generalfile.test.setup_workdir import setup_workdir
-from generalfile.errors import InvalidCharacterError
+
+
+def _thread_test(queue, i):
+    queue.put(int(Path("test.txt").write(i, overwrite=True)))
 
 
 class PathTest(unittest.TestCase):
     def setUp(self):
         """Set working dir and clear folder. Set path delimiter to '/' for testing."""
-        Path.path_delimiter = "/"
         setup_workdir()
 
 
 class FileTest(PathTest):
+    """ Skipped: open_folder, view, scrub"""
     def test_path(self):
         self.assertRaises(InvalidCharacterError, Path, "hello:there")
         self.assertRaises(InvalidCharacterError, Path, "hello<")
@@ -64,15 +67,27 @@ class FileTest(PathTest):
     def test_suffixes(self):
         path = Path("folder/test.hello.txt")
         self.assertEqual([".hello", ".txt"], path.suffixes())
-        self.assertEqual("folder/test.tsv", path.with_suffixes([".tsv"]))
+        self.assertEqual("folder/test.tsv", path.with_suffixes(".tsv"))
 
     def test_suffix(self):
+        path = Path("folder/test")
+        self.assertEqual("", path.suffix())
+
         path = Path("folder/test.txt")
         self.assertEqual(".txt", path.suffix())
 
+        path = path.with_suffix("")
+        self.assertEqual("folder/test", path)
+
+        path = path.with_suffix(None)
+        self.assertEqual("folder/test", path)
+
         path = path.with_suffix(".tsv")
         self.assertEqual("folder/test.tsv", path)
 
+        path = path.with_suffix("")
+        self.assertEqual("folder/test", path)
+
         path = path.with_suffix(".csv")
         self.assertEqual("folder/test.csv", path)
 
@@ -107,11 +122,15 @@ class FileTest(PathTest):
         path = Path("folder/foobar/test.txt")
         self.assertEqual(Path("folder/foobar"), path.get_parent())
         self.assertEqual(Path("folder/foobar"), path.get_parent(0))
-        self.assertEqual(Path("folder"), path.get_parent(1))
-        self.assertEqual(Path(), path.get_parent(2))
-        self.assertEqual(None, path.get_parent(3))
-        self.assertEqual(None, path.get_parent(99))
-        self.assertEqual(None, path.get_parent(-99))
+        self.assertEqual(Path("folder"), path.get_parent(1, 1))
+        self.assertEqual(Path(), path.get_parent(2, 2))
+        self.assertEqual(None, path.get_parent(3, 3))
+        self.assertEqual(None, path.get_parent(99, 99))
+        self.assertEqual(None, path.get_parent(-99, -99))
+
+        self.assertEqual([Path("folder/foobar"), Path("folder"), Path()], path.get_parents(depth=-1))
+        new_path = Path("folder/foobar/test.txt")
+        self.assertEqual([Path("folder/foobar"), Path("folder"), Path()], new_path.get_parents(depth=-1))
 
     def test_startswith(self):
         self.assertFalse(Path("file.txt").startswith("folder"))
@@ -144,6 +163,7 @@ class FileTest(PathTest):
         if Path.verInfo.pathRootIsDelimiter:
             self.assertEqual(Path("test.txt"), Path("folder/test.txt").remove_start("folder"))
 
+
     def test_remove_end(self):
         self.assertEqual(Path(), Path("test.txt").remove_end("test.txt"))
         self.assertEqual(Path("test"), Path("test.txt").remove_end(".txt"), "test")
@@ -163,13 +183,17 @@ class FileTest(PathTest):
         self.assertEqual(False, path.is_absolute())
         self.assertEqual(True, path.is_relative())
 
-
         path = Path("folder/folder2/file.txt")
         self.assertEqual(Path("folder2/file.txt"), path.relative("folder"))
         self.assertEqual(path.relative("folder"), "folder2/file.txt")
         self.assertEqual(path.relative("folder/folder2"), "file.txt")
 
+        self.assertEqual(path, path.relative("doesntexist"))
 
+    def test_mirror_path(self):
+        path = Path("foo")
+        self.assertEqual(path.mirror_path().mirror_path(), path)
+        self.assertEqual(True, path.mirror_path().is_absolute())
 
     def test_is_file_or_folder(self):
         Path("folder.txt/file.txt").write()
@@ -193,7 +217,6 @@ class FileTest(PathTest):
         self.assertEqual(False, path.exists())
         self.assertEqual(False, Path("folder").exists())
 
-
     def test_working_dir(self):
         self.assertEqual(True, Path.get_working_dir().is_absolute())
         self.assertEqual(Path().absolute(), Path.get_working_dir())
@@ -244,7 +267,7 @@ class FileTest(PathTest):
         Path("folder2/foobar").rename(suffix=".test")
         self.assertTrue(Path("folder2/foobar.test").exists())
 
-        Path("folder2/foobar.test").rename(stem="hello")
+        self.assertEqual("folder2/hello.test", Path("folder2/foobar.test").rename(stem="hello"))
         self.assertTrue(Path("folder2/hello.test").exists())
 
     def test_copy(self):
@@ -347,19 +370,21 @@ class FileTest(PathTest):
         Path("folder/test2.txt").write()
         Path("folder/test3.txt").write()
 
-        self.assertEqual(1, len(list(Path().get_paths_recursive(depth=1))))
-        self.assertEqual(2, len(list(Path().get_paths_recursive(depth=1, include_self=True))))
+        self.assertEqual(2, len(Path().get_children()))
+        self.assertEqual(3, len(Path().get_children(include_self=True)))
 
-        self.assertEqual(0, len(list(Path("test.txt").get_paths_recursive(depth=1))))
-        self.assertEqual(1, len(list(Path("test.txt").get_paths_recursive(depth=1, include_self=True))))
-        self.assertEqual(1, len(list(Path("test.txt").get_paths_recursive(depth=1, include_folders=True))))
+        self.assertEqual(0, len(Path("test.txt").get_children()))
+        self.assertEqual(1, len(Path("test.txt").get_children(include_self=True)))
+        self.assertEqual(0, len(Path("test.txt").get_children()))
 
-        self.assertEqual(3, len(list(Path().get_paths_recursive(depth=3))))
-        self.assertEqual(4, len(list(Path().get_paths_recursive(depth=3, include_self=True))))
-        self.assertEqual(4, len(list(Path().get_paths_recursive(depth=-1, include_self=True))))
-        self.assertEqual(2, len(list(Path().get_paths_recursive(depth=1, include_self=True))))
+        self.assertEqual(4, len(Path().get_children(depth=3)))
+        self.assertEqual(5, len(Path().get_children(depth=1, include_self=True)))
+        self.assertEqual(5, len(Path().get_children(depth=-1, include_self=True)))
+        self.assertEqual(3, len(Path().get_children(depth=0, include_self=True)))
 
-        self.assertEqual(1, len(list(Path("folder/test2.txt").get_paths_recursive())))
+        self.assertEqual(0, len(Path("folder/test2.txt").get_children(depth=-1)))
+
+        self.assertEqual(["folder/test2.txt", "folder/test3.txt"], Path("folder").get_children())
 
     def test_time_created_and_modified(self):
         path = Path("test.txt")
@@ -388,11 +413,14 @@ class FileTest(PathTest):
         self.assertIn("foo", Path("foobar"))
 
     def test_root(self):
-        str_path = Path().absolute().get_parent(-1).path
+        str_path = Path().absolute().get_parent(depth=-1, index=-1).path
         if Path.verInfo.pathRootIsDelimiter:
             self.assertEqual("/", str_path)
         else:
             self.assertTrue(len(str_path) == 3 and str_path[1] == ":" and str_path[2] == Path.path_delimiter)
+        self.assertEqual(True, Path().absolute().get_parent(-1, -1).is_root())
+        self.assertEqual(False, Path("foo").is_root())
+        self.assertEqual(False, Path().absolute().is_root())
 
     def test_as_working_dir(self):
         working_dir = Path.get_working_dir()
@@ -406,11 +434,15 @@ class FileTest(PathTest):
         self.assertEqual(True, Path("hello/there").match("*"))
         self.assertEqual(True, Path("hello/there").match("*h*"))
         self.assertEqual(True, Path(".git").match(".*"))
+        self.assertEqual(True, Path(".git").match("."))
+        self.assertEqual(True, Path("hello/there").match("The"))
+        self.assertEqual(True, Path("hello/there").match("hello/there"))
+        self.assertEqual(True, Path("foo/bar/hi").match("/bar/"))
+        self.assertEqual(True, Path("foo/bar/hi").match("\\bar\\"))
 
-        self.assertEqual(False, Path(".git").match("."))
+        self.assertEqual(False, Path("hello/there").match("x"))
         self.assertEqual(False, Path("hello/there").match("*x*"))
-        self.assertEqual(False, Path("hello/there").match("The"))
-        self.assertEqual(False, Path("hello/there").match("hello/there"))
+        self.assertEqual(False, Path("hello/there").match("there/"))
 
     def test_encode(self):
         self.assertEqual("foo/bar", Path("foo\\bar").encode())
@@ -437,21 +469,111 @@ class FileTest(PathTest):
 
         self.assertEqual(len(results), count)
 
+    def test_CaseSensitivityError(self):
+        Path("foo.txt").write("hi")
+        # Path("foo.txt").get_parent()
+        self.assertRaises(CaseSensitivityError, Path("Foo.txt").exists)
 
-def _thread_test(queue, i):
-    queue.put(int(Path("test.txt").write(i, overwrite=True)))
-
-if __name__ == "__main__":
-    x = unittest.main()
-
-
-
-
-
-
-
-
+    def test_get_alternative_path(self):
+        path = Path("foo/bar.txt")
+        self.assertEqual(path, path.to_alternative().from_alternative())
 
+        path = path.absolute()
+        self.assertEqual(path, path.to_alternative().from_alternative())
+
+    def test_get_cache_dir(self):
+        self.assertEqual(True, Path.get_lock_dir().startswith(Path.get_cache_dir()))
+
+    def test_lock(self):
+        path = Path("foo.txt")
+        with path.lock():
+            self.assertEqual(True, path.get_lock_path().exists())
+
+    def test_open_operation(self):
+        path = Path("foo.txt")
+        with path.lock():
+            path.open_operation("w", lambda stream: stream.write("hi"))
+            self.assertEqual("hi", path.open_operation("r", lambda stream: stream.read()))
+
+    def test_size(self):
+        path = Path("foo.txt")
+        path.write("bar")
+        self.assertEqual(True, path.size() > 1)
+
+    def test_without_file(self):
+        path = Path("foo/bar")
+        self.assertEqual("foo/bar", path.without_file())
+        path.write()
+        self.assertEqual("foo", path.without_file())
+
+    def test_get_differing_files(self):
+        Path("one/bar").write("hey")
+        Path("one/foo").write("hello")
+        Path("two/foo").write("hi")
+
+        for base, target in (("one", "two"), ("two", "one")):
+            self.assertEqual({"bar"}, Path(base).get_differing_files(target, exist=True, content=False))
+            self.assertEqual({"foo"}, Path(base).get_differing_files(target, exist=False, content=True))
+            self.assertEqual({"foo", "bar"}, Path(base).get_differing_files(target, exist=True, content=True))
+            self.assertEqual(set(), Path(base).get_differing_files(target, exist=False, content=False))
+
+    def test_contains(self):
+        path = Path("foo")
+        path.text.write("hello there test")
+        self.assertEqual(True, path.contains("there"))
+        self.assertEqual(False, path.contains("hi"))
+
+    def test_is_identical(self):
+        Path("foo").write("hello")
+        Path("bar").write("hello")
+        self.assertEqual(True, Path("foo").is_identical("bar"))
+
+        Path("bar").write("hi", overwrite=True)
+        self.assertEqual(False, Path("foo").is_identical("bar"))
+
+        Path("foo").write("hi\n", overwrite=True)
+        self.assertEqual(False, Path("foo").is_identical("bar"))
+
+    def test_empty(self):
+        self.assertEqual(True, Path().empty())
+        self.assertEqual(True, Path("new").empty())
+        Path("new").create_folder()
+        self.assertEqual(True, Path("new").empty())
+        Path("new/file.txt").write("foo")
+        self.assertEqual(False, Path("new").empty())
+        self.assertEqual(False, Path("new/file.txt").empty())
+        Path("new/file.txt").delete()
+        self.assertEqual(True, Path("new").empty())
+
+    def test_pack(self):
+        Path("base/test.txt").write("hello")
+        Path("base").pack("target")
+        self.assertEqual(True, Path("target.zip").exists())
+        Path("target").unpack("new")
+        self.assertEqual("hello", Path("new/test.txt").read())
+
+        Path("base").pack("target.tar.gz").unpack("tarnew")
+        self.assertEqual("hello", Path("tarnew/test.txt").read())
+
+        Path("base/folder/hi").write("there")
+        Path("base").pack("packed/pack.zip").unpack("newbase")
+        self.assertEqual("hello", Path("newbase/test.txt").read())
+        self.assertEqual("there", Path("newbase/folder/hi").read())
+
+    def test_recycle(self):
+        self.assertIs(Path("hi/there"), Path("hi/there"))
+        self.assertIs(Path("hi/there")._children, Path("hi/there")._children)
+        self.assertIs(Path("hi/there"), Path("hi\\there"))
+        self.assertIs(Path("hi/there"), Path("hi") / "there")
+        self.assertIs(Path("hi\\there"), Path("hi") / "there")
+
+        self.assertIsNot(Path("hithere"), Path("hi") / "there")
+        self.assertIsNot(Path("hi"), Path("hi").absolute())
+
+    def test_read_empty(self):
+        with self.assertRaises(FileNotFoundError):
+            Path("hey").read()
+        self.assertEqual(None, Path("hi").read(default=None))
 
 
 
